@@ -1,5 +1,7 @@
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -61,8 +63,9 @@ class MessagesController < ApplicationController
     if request.post?
       @message.save_attachments(params[:attachments])
       if @message.save
-        call_hook(:controller_messages_new_after_save, { :params => params, :message => @message})
+        call_hook(:controller_messages_new_after_save, {:params => params, :message => @message})
         render_attachment_warning_if_needed(@message)
+        flash[:notice] = l(:notice_successful_create)
         redirect_to board_message_path(@board, @message)
       end
     end
@@ -74,12 +77,13 @@ class MessagesController < ApplicationController
     @reply.author = User.current
     @reply.board = @board
     @reply.safe_attributes = params[:reply]
+    @reply.save_attachments(params[:attachments])
     @topic.children << @reply
     if !@reply.new_record?
-      call_hook(:controller_messages_reply_after_save, { :params => params, :message => @reply})
-      attachments = Attachment.attach_files(@reply, params[:attachments])
+      call_hook(:controller_messages_reply_after_save, {:params => params, :message => @reply})
       render_attachment_warning_if_needed(@reply)
     end
+    flash[:notice] = l(:notice_successful_update)
     redirect_to board_message_path(@board, @topic, :r => @reply)
   end
 
@@ -87,12 +91,14 @@ class MessagesController < ApplicationController
   def edit
     (render_403; return false) unless @message.editable_by?(User.current)
     @message.safe_attributes = params[:message]
-    if request.post? && @message.save
-      attachments = Attachment.attach_files(@message, params[:attachments])
-      render_attachment_warning_if_needed(@message)
-      flash[:notice] = l(:notice_successful_update)
-      @message.reload
-      redirect_to board_message_path(@message.board, @message.root, :r => (@message.parent_id && @message.id))
+    if request.post?
+      @message.save_attachments(params[:attachments])
+      if @message.save
+        render_attachment_warning_if_needed(@message)
+        flash[:notice] = l(:notice_successful_update)
+        @message.reload
+        redirect_to board_message_path(@message.board, @message.root, :r => (@message.parent_id && @message.id))
+      end
     end
   end
 
@@ -101,6 +107,7 @@ class MessagesController < ApplicationController
     (render_403; return false) unless @message.destroyable_by?(User.current)
     r = @message.to_param
     @message.destroy
+    flash[:notice] = l(:notice_successful_delete)
     if @message.parent
       redirect_to board_message_path(@board, @message.parent, :r => r)
     else
@@ -112,20 +119,31 @@ class MessagesController < ApplicationController
     @subject = @message.subject
     @subject = "RE: #{@subject}" unless @subject.starts_with?('RE:')
 
-    @content = "#{ll(Setting.default_language, :text_user_wrote, @message.author)}\n> "
+    if @message.root == @message
+      @content = +"#{ll(Setting.default_language, :text_user_wrote, @message.author)}\n> "
+    else
+      @content = +"#{ll(Setting.default_language, :text_user_wrote_in, {:value => @message.author, :link => "message##{@message.id}"})}\n> "
+    end
     @content << @message.content.to_s.strip.gsub(%r{<pre>(.*?)</pre>}m, '[...]').gsub(/(\r?\n|\r\n?)/, "\n> ") + "\n\n"
+
+    respond_to do |format|
+      format.html { render_404 }
+      format.js
+    end
   end
 
   def preview
     message = @board.messages.find_by_id(params[:id])
-    @text = (params[:message] || params[:reply])[:content]
+    @text = params[:text] ? params[:text] : nil
     @previewed = message
     render :partial => 'common/preview'
   end
 
-private
+  private
+
   def find_message
     return unless find_board
+
     @message = @board.messages.includes(:parent).find(params[:id])
     @topic = @message.root
   rescue ActiveRecord::RecordNotFound
