@@ -1,5 +1,7 @@
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,24 +17,34 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
+require 'redmine/my_page'
+
 class UserPreference < ActiveRecord::Base
   include Redmine::SafeAttributes
 
   belongs_to :user
   serialize :others
 
-  attr_protected :others, :user_id
-
   before_save :set_others_hash, :clear_unused_block_settings
 
-  safe_attributes 'hide_mail',
+  safe_attributes(
+    'hide_mail',
     'time_zone',
     'comments_sorting',
     'warn_on_leaving_unsaved',
     'no_self_notified',
-    'textarea_font'
+    'notify_about_high_priority_issues',
+    'textarea_font',
+    'recently_used_projects',
+    'history_default_tab',
+    'default_issue_query',
+    'default_project_query',
+    'toolbar_language_options',
+    'auto_watch_on')
 
   TEXTAREA_FONT_OPTIONS = ['monospace', 'proportional']
+  DEFAULT_TOOLBAR_LANGUAGE_OPTIONS = %w[c cpp csharp css diff go groovy html java javascript objc perl php python r ruby sass scala shell sql swift xml yaml]
+  AUTO_WATCH_ON_OPTIONS = %w[issue_created issue_contributed_to]
 
   def initialize(attributes=nil, *args)
     super
@@ -44,7 +56,10 @@ class UserPreference < ActiveRecord::Base
         self.time_zone = Setting.default_users_time_zone
       end
       unless attributes && attributes.key?(:no_self_notified)
-        self.no_self_notified = true
+        self.no_self_notified = Setting.default_users_no_self_notified
+      end
+      unless attributes && attributes.key?(:auto_watch_on)
+        self.auto_watch_on = AUTO_WATCH_ON_OPTIONS
       end
     end
     self.others ||= {}
@@ -73,8 +88,8 @@ class UserPreference < ActiveRecord::Base
     end
   end
 
-  def comments_sorting; self[:comments_sorting] end
-  def comments_sorting=(order); self[:comments_sorting]=order end
+  def comments_sorting; self[:comments_sorting]; end
+  def comments_sorting=(order); self[:comments_sorting]=order; end
 
   def warn_on_leaving_unsaved; self[:warn_on_leaving_unsaved] || '1'; end
   def warn_on_leaving_unsaved=(value); self[:warn_on_leaving_unsaved]=value; end
@@ -82,12 +97,42 @@ class UserPreference < ActiveRecord::Base
   def no_self_notified; (self[:no_self_notified] == true || self[:no_self_notified] == '1'); end
   def no_self_notified=(value); self[:no_self_notified]=value; end
 
-  def activity_scope; Array(self[:activity_scope]) ; end
-  def activity_scope=(value); self[:activity_scope]=value ; end
+  def notify_about_high_priority_issues; (self[:notify_about_high_priority_issues] == true || self[:notify_about_high_priority_issues] == '1'); end
+  def notify_about_high_priority_issues=(value); self[:notify_about_high_priority_issues]=value; end
 
-  def textarea_font; self[:textarea_font] end
+  def activity_scope; Array(self[:activity_scope]); end
+  def activity_scope=(value); self[:activity_scope]=value; end
+
+  def textarea_font; self[:textarea_font]; end
   def textarea_font=(value); self[:textarea_font]=value; end
 
+  def recently_used_projects; (self[:recently_used_projects] || 3).to_i; end
+  def recently_used_projects=(value); self[:recently_used_projects] = value.to_i; end
+  def history_default_tab; self[:history_default_tab]; end
+  def history_default_tab=(value); self[:history_default_tab]=value; end
+
+  def toolbar_language_options
+    self[:toolbar_language_options].presence || DEFAULT_TOOLBAR_LANGUAGE_OPTIONS.join(',')
+  end
+
+  def toolbar_language_options=(value)
+    languages =
+      value.to_s.delete(' ').split(',').select do |lang|
+        Redmine::SyntaxHighlighting.language_supported?(lang)
+      end.compact
+    self[:toolbar_language_options] = languages.join(',')
+  end
+
+  def default_issue_query; self[:default_issue_query] end
+  def default_issue_query=(value); self[:default_issue_query]=value; end
+
+  def default_project_query; self[:default_project_query] end
+  def default_project_query=(value); self[:default_project_query]=value; end
+
+  def auto_watch_on; self[:auto_watch_on] || []; end
+  def auto_watch_on=(values); self[:auto_watch_on]=values; end
+  def auto_watch_on?(action); self.auto_watch_on.include?(action.to_s); end
+
   # Returns the names of groups that are displayed on user's page
   # Example:
   #   preferences.my_page_groups
@@ -122,7 +167,7 @@ class UserPreference < ActiveRecord::Base
   #   preferences.remove_block('news')
   def remove_block(block)
     block = block.to_s.underscore
-    my_page_layout.keys.each do |group|
+    my_page_layout.each_key do |group|
       my_page_layout[group].delete(block)
     end
     my_page_layout
