@@ -1,10 +1,13 @@
-require 'rexml/document'
+# frozen_string_literal: true
+
+require 'redmine/scm/adapters/subversion_adapter'
 
 module Redmine
-  module VERSION #:nodoc:
-    MAJOR = 3
-    MINOR = 4
-    TINY  = 6
+  # @private
+  module VERSION
+    MAJOR = 5
+    MINOR = 1
+    TINY  = 11
 
     # Branch values:
     # * official release: nil
@@ -17,7 +20,7 @@ module Redmine
       if File.directory?(File.join(Rails.root, '.svn'))
         begin
           path = Redmine::Scm::Adapters::AbstractAdapter.shell_quote(Rails.root.to_s)
-          if `svn info --xml #{path}` =~ /revision="(\d+)"/
+          if `#{Redmine::Scm::Adapters::SubversionAdapter.client_command} info --xml #{path}` =~ /commit\s+revision="(\d+)"/
             return $1.to_i
           end
         rescue
