@@ -1,5 +1,7 @@
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2016  Jean-Philippe Lang
+# Copyright (C) 2006-  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -23,9 +25,19 @@ module Redmine
       end
 
       def link_to(name, options={})
-        url = { :format => name.to_s.downcase }.merge(options.delete(:url) || {}).except('page')
+        url = {:format => name.to_s.downcase}.merge(options.delete(:url) || {}).except('page')
         caption = options.delete(:caption) || name
-        html_options = { :class => name.to_s.downcase, :rel => 'nofollow' }.merge(options)
+        html_options = {:class => name.to_s.downcase, :rel => 'nofollow'}.merge(options)
+        @view.content_tag('span', @view.link_to(caption, url, html_options))
+      end
+
+      # Preserves query parameters
+      def link_to_with_query_parameters(name, url={}, options={})
+        params = @view.request.query_parameters.except(:page, :format).except(*url.keys)
+        url = {:params => params, :page => nil, :format => name.to_s.downcase}.merge(url)
+
+        caption = options.delete(:caption) || name
+        html_options = {:class => name.to_s.downcase, :rel => 'nofollow'}.merge(options)
         @view.content_tag('span', @view.link_to(caption, url, html_options))
       end
     end
