@@ -1,5 +1,7 @@
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,10 +26,16 @@ module Redmine
           Redmine::WikiFormatting::Macros.available_macros.key?(name.to_sym)
         end
 
-        def exec_macro(name, obj, args, text)
+        def exec_macro(name, obj, args, text, options={})
           macro_options = Redmine::WikiFormatting::Macros.available_macros[name.to_sym]
           return unless macro_options
 
+          if options[:inline_attachments] == false
+            Redmine::WikiFormatting::Macros.inline_attachments = false
+          else
+            Redmine::WikiFormatting::Macros.inline_attachments = true
+          end
+
           method_name = "macro_#{name}"
           unless macro_options[:parse_args] == false
             args = args.split(',').map(&:strip)
@@ -37,12 +45,12 @@ module Redmine
             if self.class.instance_method(method_name).arity == 3
               send(method_name, obj, args, text)
             elsif text
-              raise "This macro does not accept a block of text"
+              raise t(:error_macro_does_not_accept_block)
             else
               send(method_name, obj, args)
             end
           rescue => e
-            "<div class=\"flash error\">Error executing the <strong>#{h name}</strong> macro (#{h e.to_s})</div>".html_safe
+            %|<div class="flash error">#{::I18n.t(:error_can_not_execute_macro_html, :name => ::ERB::Util.h(name), :error => ::ERB::Util.h(e.to_s))}</div>|.html_safe
           end
         end
 
@@ -57,7 +65,9 @@ module Redmine
       end
 
       @@available_macros = {}
+      @@inline_attachments = true
       mattr_accessor :available_macros
+      mattr_accessor :inline_attachments
 
       class << self
         # Plugins can use this method to define new macros:
@@ -67,21 +77,21 @@ module Redmine
         #     macro :my_macro do |obj, args|
         #       "My macro output"
         #     end
-        #   
+        #
         #     desc "This is my macro that accepts a block of text"
         #     macro :my_macro do |obj, args, text|
         #       "My macro output"
         #     end
         #   end
         def register(&block)
-          class_eval(&block) if block_given?
+          class_eval(&block) if block
         end
 
         # Defines a new macro with the given name, options and block.
         #
         # Options:
         # * :desc - A description of the macro
-        # * :parse_args => false - Disables arguments parsing (the whole arguments 
+        # * :parse_args => false - Disables arguments parsing (the whole arguments
         #   string is passed to the macro)
         #
         # Macro blocks accept 2 or 3 arguments:
@@ -89,7 +99,7 @@ module Redmine
         # * args: macro arguments
         # * text: the block of text given to the macro (should be present only if the
         #   macro accepts a block of text). text is a String or nil if the macro is
-        #   invoked without a block of text.  
+        #   invoked without a block of text.
         #
         # Examples:
         # By default, when the macro is invoked, the comma separated list of arguments
@@ -141,12 +151,13 @@ module Redmine
         # If a block of text is given, the closing tag }} must be at the start of a new line.
         def macro(name, options={}, &block)
           options.assert_valid_keys(:desc, :parse_args)
-          unless name.to_s.match(/\A\w+\z/)
+          unless /\A\w+\z/.match?(name.to_s)
             raise "Invalid macro name: #{name} (only 0-9, A-Z, a-z and _ characters are accepted)"
           end
-          unless block_given?
+          unless block
             raise "Can not create a macro without a block!"
           end
+
           name = name.to_s.downcase.to_sym
           available_macros[name] = {:desc => @@desc || ''}.merge(options)
           @@desc = nil
@@ -162,7 +173,8 @@ module Redmine
       # Builtin macros
       desc "Sample macro."
       macro :hello_world do |obj, args, text|
-        h("Hello world! Object: #{obj.class.name}, " + 
+        h(
+          "Hello world! Object: #{obj.class.name}, " +
           (args.empty? ? "Called with no argument" : "Arguments: #{args.join(', ')}") +
           " and " + (text.present? ? "a #{text.size} bytes long block of text." : "no block of text.")
         )
@@ -190,12 +202,13 @@ module Redmine
         page = nil
         if args.size > 0
           page = Wiki.find_page(args.first.to_s, :project => @project)
-        elsif obj.is_a?(WikiContent) || obj.is_a?(WikiContent::Version)
+        elsif obj.is_a?(WikiContent) || obj.is_a?(WikiContentVersion)
           page = obj.page
         else
-          raise 'With no argument, this macro can be called from wiki pages only.'
+          raise t(:error_childpages_macro_no_argument)
         end
-        raise 'Page not found' if page.nil? || !User.current.allowed_to?(:view_wiki_pages, page.wiki.project)
+        raise t(:error_page_not_found) if page.nil? || !User.current.allowed_to?(:view_wiki_pages, page.wiki.project)
+
         pages = page.self_and_descendants(options[:depth]).group_by(&:parent_id)
         render_page_hierarchy(pages, options[:parent] ? page.parent_id : page.id)
       end
@@ -205,11 +218,20 @@ module Redmine
              "{{include(projectname:Foo)}} -- to include a page of a specific project wiki"
       macro :include do |obj, args|
         page = Wiki.find_page(args.first.to_s, :project => @project)
-        raise 'Page not found' if page.nil? || !User.current.allowed_to?(:view_wiki_pages, page.wiki.project)
+        raise t(:error_page_not_found) if page.nil? || !User.current.allowed_to?(:view_wiki_pages, page.wiki.project)
+
         @included_wiki_pages ||= []
-        raise 'Circular inclusion detected' if @included_wiki_pages.include?(page.id)
+        raise t(:error_circular_inclusion) if @included_wiki_pages.include?(page.id)
+
         @included_wiki_pages << page.id
-        out = textilizable(page.content, :text, :attachments => page.attachments, :headings => false)
+        out =
+          textilizable(
+            page.content,
+            :text,
+            :attachments => page.attachments,
+            :headings => false,
+            :inline_attachments => @@inline_attachments
+          )
         @included_wiki_pages.pop
         out
       end
@@ -223,32 +245,84 @@ module Redmine
         hide_label = args[1] || args[0] || l(:button_hide)
         js = "$('##{html_id}-show, ##{html_id}-hide').toggle(); $('##{html_id}').fadeToggle(150);"
         out = ''.html_safe
-        out << link_to_function(show_label, js, :id => "#{html_id}-show", :class => 'collapsible collapsed')
-        out << link_to_function(hide_label, js, :id => "#{html_id}-hide", :class => 'collapsible', :style => 'display:none;')
-        out << content_tag('div', textilizable(text, :object => obj, :headings => false), :id => html_id, :class => 'collapsed-text', :style => 'display:none;')
+        out << link_to_function(show_label, js, :id => "#{html_id}-show", :class => 'icon icon-collapsed collapsible')
+        out <<
+          link_to_function(
+            hide_label, js,
+            :id => "#{html_id}-hide",
+            :class => 'icon icon-expanded collapsible',
+            :style => 'display:none;'
+          )
+        out <<
+          content_tag(
+            'div',
+            textilizable(text, :object => obj, :headings => false,
+                         :inline_attachments => @@inline_attachments),
+            :id => html_id, :class => 'collapsed-text',
+            :style => 'display:none;'
+          )
         out
       end
 
-      desc "Displays a clickable thumbnail of an attached image. Examples:\n\n" +
+      desc "Displays a clickable thumbnail of an attached image.\n" +
+             "Default size is 200 pixels. Examples:\n\n" +
              "{{thumbnail(image.png)}}\n" +
              "{{thumbnail(image.png, size=300, title=Thumbnail)}} -- with custom title and size"
       macro :thumbnail do |obj, args|
         args, options = extract_macro_options(args, :size, :title)
         filename = args.first
-        raise 'Filename required' unless filename.present?
-        size = options[:size]
-        raise 'Invalid size parameter' unless size.nil? || size.match(/^\d+$/)
-        size = size.to_i
-        size = nil unless size > 0
-        if obj && obj.respond_to?(:attachments) && attachment = Attachment.latest_attach(obj.attachments, filename)
-          title = options[:title] || attachment.title
-          thumbnail_url = url_for(:controller => 'attachments', :action => 'thumbnail', :id => attachment, :size => size, :only_path => @only_path)
-          image_url = url_for(:controller => 'attachments', :action => 'show', :id => attachment, :only_path => @only_path)
+        raise t(:error_filename_required) unless filename.present?
 
+        size = options[:size]
+        raise t(:error_invalid_size_parameter) unless size.nil? || /^\d+$/.match?(size)
+
+        size = size.to_i
+        size = 200 unless size > 0
+
+        container = obj.is_a?(Journal) ? obj.journalized : obj
+        attachments = container.attachments if container.respond_to?(:attachments)
+        if (controller_path == 'previews' || action_name == 'preview') && @attachments.present?
+          attachments = (attachments.to_a + @attachments).compact
+        end
+        if attachments.present? && (attachment = Attachment.latest_attach(attachments, filename))
+          title = options[:title] || attachment.title
+          thumbnail_url =
+            url_for(:controller => 'attachments', :action => 'thumbnail',
+                    :id => attachment, :size => size, :only_path => @only_path)
+          image_url =
+            url_for(:controller => 'attachments', :action => 'show',
+                    :id => attachment, :only_path => @only_path)
           img = image_tag(thumbnail_url, :alt => attachment.filename)
           link_to(img, image_url, :class => 'thumbnail', :title => title)
         else
-          raise "Attachment #{filename} not found"
+          raise t(:error_attachment_not_found, :name => filename)
+        end
+      end
+
+      desc "Displays an issue link including additional information. Examples:\n\n" +
+             "{{issue(123)}}                              -- Issue #123: Enhance macro capabilities\n" +
+             "{{issue(123, project=true)}}                -- Andromeda - Issue #123: Enhance macro capabilities\n" +
+             "{{issue(123, tracker=false)}}               -- #123: Enhance macro capabilities\n" +
+             "{{issue(123, subject=false, project=true)}} -- Andromeda - Issue #123\n"
+      macro :issue do |obj, args|
+        args, options = extract_macro_options(args, :project, :subject, :tracker)
+        id = args.first
+        issue = Issue.visible.find_by(id: id)
+
+        if issue
+          # remove invalid options
+          options.delete_if {|k, v| v != 'true' && v != 'false'}
+
+          # turn string values into boolean
+          options.each do |k, v|
+            options[k] = v == 'true'
+          end
+
+          link_to_issue(issue, options)
+        else
+          # Fall back to regular issue link format to indicate, that there
+          # should have been something.
+          "##{id}"
         end
       end
     end
