@@ -1,5 +1,7 @@
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,11 +17,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-require File.expand_path('../../test_helper', __FILE__)
+require_relative '../test_helper'
 
 class AuthSourcesControllerTest < Redmine::ControllerTest
-  fixtures :users, :auth_sources
-
   def setup
     @request.session[:user_id] = 1
   end
@@ -32,6 +32,7 @@ class AuthSourcesControllerTest < Redmine::ControllerTest
   def test_new
     get :new
     assert_response :success
+    assert_includes @response.headers['Cache-Control'], 'no-store'
 
     assert_select 'form#auth_source_form' do
       assert_select 'input[name=type][value=AuthSourceLdap]'
@@ -40,15 +41,20 @@ class AuthSourcesControllerTest < Redmine::ControllerTest
   end
 
   def test_new_with_invalid_type_should_respond_with_404
-    get :new, :params => {
+    get(
+      :new,
+      :params => {
         :type => 'foo'
       }
+    )
     assert_response 404
   end
 
   def test_create
     assert_difference 'AuthSourceLdap.count' do
-      post :create, :params => {
+      post(
+        :create,
+        :params => {
           :type => 'AuthSourceLdap',
           :auth_source => {
             :name => 'Test',
@@ -57,6 +63,7 @@ class AuthSourcesControllerTest < Redmine::ControllerTest
             :attr_login => 'cn'
           }
         }
+      )
       assert_redirected_to '/auth_sources'
     end
 
@@ -69,7 +76,9 @@ class AuthSourcesControllerTest < Redmine::ControllerTest
 
   def test_create_with_failure
     assert_no_difference 'AuthSourceLdap.count' do
-      post :create, :params => {
+      post(
+        :create,
+        :params => {
           :type => 'AuthSourceLdap',
           :auth_source => {
             :name => 'Test',
@@ -78,16 +87,22 @@ class AuthSourcesControllerTest < Redmine::ControllerTest
             :attr_login => 'cn'
           }
         }
+      )
       assert_response :success
+      assert_includes @response.headers['Cache-Control'], 'no-store'
     end
     assert_select_error /host cannot be blank/i
   end
 
   def test_edit
-    get :edit, :params => {
+    get(
+      :edit,
+      :params => {
         :id => 1
       }
+    )
     assert_response :success
+    assert_includes @response.headers['Cache-Control'], 'no-store'
 
     assert_select 'form#auth_source_form' do
       assert_select 'input[name=?]', 'auth_source[host]'
@@ -97,23 +112,31 @@ class AuthSourcesControllerTest < Redmine::ControllerTest
   def test_edit_should_not_contain_password
     AuthSource.find(1).update_column :account_password, 'secret'
 
-    get :edit, :params => {
+    get(
+      :edit,
+      :params => {
         :id => 1
       }
+    )
     assert_response :success
     assert_select 'input[value=secret]', 0
     assert_select 'input[name=dummy_password][value^=xxxxxx]'
   end
 
   def test_edit_invalid_should_respond_with_404
-    get :edit, :params => {
+    get(
+      :edit,
+      :params => {
         :id => 99
       }
+    )
     assert_response 404
   end
 
   def test_update
-    put :update, :params => {
+    put(
+      :update,
+      :params => {
         :id => 1,
         :auth_source => {
           :name => 'Renamed',
@@ -122,6 +145,7 @@ class AuthSourcesControllerTest < Redmine::ControllerTest
           :attr_login => 'uid'
         }
       }
+    )
     assert_redirected_to '/auth_sources'
     source = AuthSourceLdap.find(1)
     assert_equal 'Renamed', source.name
@@ -129,7 +153,9 @@ class AuthSourcesControllerTest < Redmine::ControllerTest
   end
 
   def test_update_with_failure
-    put :update, :params => {
+    put(
+      :update,
+      :params => {
         :id => 1,
         :auth_source => {
           :name => 'Renamed',
@@ -138,15 +164,21 @@ class AuthSourcesControllerTest < Redmine::ControllerTest
           :attr_login => 'uid'
         }
       }
+    )
     assert_response :success
+    assert_includes @response.headers['Cache-Control'], 'no-store'
+
     assert_select_error /host cannot be blank/i
   end
 
   def test_destroy
     assert_difference 'AuthSourceLdap.count', -1 do
-      delete :destroy, :params => {
+      delete(
+        :destroy,
+        :params => {
           :id => 1
         }
+      )
       assert_redirected_to '/auth_sources'
     end
   end
@@ -155,46 +187,62 @@ class AuthSourcesControllerTest < Redmine::ControllerTest
     User.find(2).update_attribute :auth_source_id, 1
 
     assert_no_difference 'AuthSourceLdap.count' do
-      delete :destroy, :params => {
+      delete(
+        :destroy,
+        :params => {
           :id => 1
         }
+      )
       assert_redirected_to '/auth_sources'
+      assert_equal 'This authentication mode is in use and cannot be deleted.', flash[:error]
     end
   end
 
   def test_test_connection
     AuthSourceLdap.any_instance.stubs(:test_connection).returns(true)
 
-    get :test_connection, :params => {
+    get(
+      :test_connection,
+      :params => {
         :id => 1
       }
+    )
     assert_redirected_to '/auth_sources'
     assert_not_nil flash[:notice]
     assert_match /successful/i, flash[:notice]
   end
 
   def test_test_connection_with_failure
-    AuthSourceLdap.any_instance.stubs(:initialize_ldap_con).raises(Net::LDAP::LdapError.new("Something went wrong"))
+    AuthSourceLdap.any_instance.stubs(:initialize_ldap_con).raises(Net::LDAP::Error.new("Something went wrong"))
 
-    get :test_connection, :params => {
+    get(
+      :test_connection,
+      :params => {
         :id => 1
       }
+    )
     assert_redirected_to '/auth_sources'
     assert_not_nil flash[:error]
     assert_include 'Something went wrong', flash[:error]
   end
 
   def test_autocomplete_for_new_user
-    AuthSource.expects(:search).with('foo').returns([
-      {:login => 'foo1', :firstname => 'John', :lastname => 'Smith', :mail => 'foo1@example.net', :auth_source_id => 1},
-      {:login => 'Smith', :firstname => 'John', :lastname => 'Doe', :mail => 'foo2@example.net', :auth_source_id => 1}
-    ])
-
-    get :autocomplete_for_new_user, :params => {
+    AuthSource.expects(:search).with('foo').returns(
+      [
+        {:login => 'foo1', :firstname => 'John', :lastname => 'Smith',
+         :mail => 'foo1@example.net', :auth_source_id => 1},
+        {:login => 'Smith', :firstname => 'John', :lastname => 'Doe',
+         :mail => 'foo2@example.net', :auth_source_id => 1}
+      ]
+    )
+    get(
+      :autocomplete_for_new_user,
+      :params => {
         :term => 'foo'
       }
+    )
     assert_response :success
-    assert_equal 'application/json', response.content_type
+    assert_equal 'application/json', response.media_type
     json = ActiveSupport::JSON.decode(response.body)
     assert_kind_of Array, json
     assert_equal 2, json.size
