@@ -1,5 +1,7 @@
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,11 +17,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-require File.expand_path('../../test_helper', __FILE__)
+require_relative '../test_helper'
 
 class WorkflowsControllerTest < Redmine::ControllerTest
-  fixtures :roles, :trackers, :workflows, :users, :issue_statuses
-
   def setup
     User.current = nil
     @request.session[:user_id] = 1 # admin
@@ -48,9 +48,10 @@ class WorkflowsControllerTest < Redmine::ControllerTest
 
     # used status only
     statuses = IssueStatus.where(:id => [2, 3, 5]).sorted.pluck(:name)
-    assert_equal ["New issue"] + statuses,
-      css_select('table.workflows.transitions-always tbody tr td:first').map(&:text).map(&:strip)
-
+    assert_equal(
+      ["New issue"] + statuses,
+      css_select('table.workflows.transitions-always tbody tr td:first').map {|e| e.text.strip}
+    )
     # allowed transitions
     assert_select 'input[type=checkbox][name=?][value="1"][checked=checked]', 'transitions[3][5][always]'
     # not allowed
@@ -74,8 +75,26 @@ class WorkflowsControllerTest < Redmine::ControllerTest
 
     # statuses 1 and 5 not displayed
     statuses = IssueStatus.where(:id => [2, 3]).sorted.pluck(:name)
-    assert_equal ["New issue"] + statuses,
-      css_select('table.workflows.transitions-always tbody tr td:first').map(&:text).map(&:strip)
+    assert_equal(
+      ["New issue"] + statuses,
+      css_select('table.workflows.transitions-always tbody tr td:first').map {|e| e.text.strip}
+    )
+  end
+
+  def test_get_edit_with_role_and_tracker_should_not_include_only_identity_workflows
+    WorkflowTransition.delete_all
+    WorkflowTransition.create!(:role_id => 1, :tracker_id => 1, :old_status_id => 1, :new_status_id => 1)
+    WorkflowTransition.create!(:role_id => 1, :tracker_id => 1, :old_status_id => 2, :new_status_id => 3)
+
+    get :edit, :params => {:role_id => 1, :tracker_id => 1}
+    assert_response :success
+
+    # statuses 1 and 5 not displayed
+    statuses = IssueStatus.where(:id => [2, 3]).sorted.pluck(:name)
+    assert_equal(
+      ["New issue"] + statuses,
+      css_select('table.workflows.transitions-always tbody tr td:first').map {|e| e.text.strip}
+    )
   end
 
   def test_get_edit_should_include_allowed_statuses_for_new_issues
@@ -92,8 +111,8 @@ class WorkflowsControllerTest < Redmine::ControllerTest
     get :edit, :params => {:role_id => 'all', :tracker_id => 'all'}
     assert_response :success
 
-    assert_select 'select[name=?][multiple=multiple]', 'role_id[]' do
-      assert_select 'option[selected=selected]', Role.all.select(&:consider_workflow?).count
+    assert_select 'select[name=?]', 'role_id[]' do
+      assert_select 'option[selected=selected][value=all]'
     end
     assert_select 'select[name=?]', 'tracker_id[]' do
       assert_select 'option[selected=selected][value=all]'
@@ -107,16 +126,29 @@ class WorkflowsControllerTest < Redmine::ControllerTest
     assert_response :success
 
     statuses = IssueStatus.all.sorted.pluck(:name)
-    assert_equal ["New issue"] + statuses,
-      css_select('table.workflows.transitions-always tbody tr td:first').map(&:text).map(&:strip)
+    assert_equal(
+      ["New issue"] + statuses,
+      css_select('table.workflows.transitions-always tbody tr td:first').map {|e| e.text.strip}
+    )
+    assert_select 'input[type=checkbox][name=?]', 'transitions[0][1][always]'
+  end
 
-    assert_select 'input[type=checkbox][name=?]', 'transitions[1][1][always]'
+  def test_get_edit_should_show_checked_disabled_transition_checkbox_between_same_statuses
+    get :edit, :params => {:role_id => 2, :tracker_id => 1}
+    assert_response :success
+    assert_select 'table.workflows.transitions-always tbody tr:nth-child(2)' do
+      assert_select 'td.name', :text => 'New'
+      # assert that the td is enabled
+      assert_select "td.enabled[title='New » New']"
+      # assert that the checkbox is disabled and checked
+      assert_select "input[name='transitions[1][1][always]'][checked=?][disabled=?]", 'checked', 'disabled', 1
+    end
   end
 
   def test_post_edit
     WorkflowTransition.delete_all
 
-    post :edit, :params => {
+    patch :update, :params => {
       :role_id => 2,
       :tracker_id => 1,
       :transitions => {
@@ -127,14 +159,14 @@ class WorkflowsControllerTest < Redmine::ControllerTest
     assert_response 302
 
     assert_equal 3, WorkflowTransition.where(:tracker_id => 1, :role_id => 2).count
-    assert_not_nil  WorkflowTransition.where(:role_id => 2, :tracker_id => 1, :old_status_id => 3, :new_status_id => 2).first
-    assert_nil      WorkflowTransition.where(:role_id => 2, :tracker_id => 1, :old_status_id => 5, :new_status_id => 4).first
+    assert          WorkflowTransition.where(:role_id => 2, :tracker_id => 1, :old_status_id => 3, :new_status_id => 2).exists?
+    assert_not      WorkflowTransition.where(:role_id => 2, :tracker_id => 1, :old_status_id => 5, :new_status_id => 4).exists?
   end
 
   def test_post_edit_with_allowed_statuses_for_new_issues
     WorkflowTransition.delete_all
 
-    post :edit, :params => {
+    patch :update, :params => {
       :role_id => 2,
       :tracker_id => 1,
       :transitions => {
@@ -151,7 +183,7 @@ class WorkflowsControllerTest < Redmine::ControllerTest
   def test_post_edit_with_additional_transitions
     WorkflowTransition.delete_all
 
-    post :edit, :params => {
+    patch :update, :params => {
       :role_id => 2,
       :tracker_id => 1,
       :transitions => {
@@ -179,6 +211,45 @@ class WorkflowsControllerTest < Redmine::ControllerTest
     assert w.assignee
   end
 
+  def test_post_edit_with_large_number_of_statuses
+    # This test ensures that workflows with many statuses can be saved.
+    # Without setting `ENV['RACK_QUERY_PARSER_PARAMS_LIMIT']`, this raises
+    # ActionController::BadRequest exception due to exceeding the default
+    # query parameter limit of 4096.
+    WorkflowTransition.delete_all
+
+    num_statuses = 40
+    transitions_data = {}
+
+    # Allowed statuses for a new issue (status_id = 0)
+    transitions_data['0'] = {}
+    (1..num_statuses).each do |status_id|
+      transitions_data['0'][status_id.to_s] = {'always' => '1'}
+    end
+
+    # Status transitions between statuses
+    (1..num_statuses).each do |status_id_from| # rubocop:disable RuboCopStyle/CombinableLoops
+      transitions_data[status_id_from.to_s] = {}
+      (1..num_statuses).each do |status_id_to|
+        # skip self-transitions
+        next if status_id_from == status_id_to
+
+        transitions_data[status_id_from.to_s][status_id_to.to_s] = {
+          'always' => '1', 'author' => '1', 'assignee' => '1'
+        }
+      end
+    end
+
+    assert_nothing_raised do
+      patch :update, :params => {
+        :role_id => 2,
+        :tracker_id => 1,
+        :transitions => transitions_data
+      }
+    end
+    assert_response :found
+  end
+
   def test_get_permissions
     get :permissions
 
@@ -307,8 +378,10 @@ class WorkflowsControllerTest < Redmine::ControllerTest
     assert_response :success
 
     statuses = IssueStatus.all.sorted.pluck(:name)
-    assert_equal statuses,
-      css_select('table.workflows.fields_permissions thead tr:nth-child(2) td:not(:first-child)').map(&:text).map(&:strip)
+    assert_equal(
+      statuses,
+      css_select('table.workflows.fields_permissions thead tr:nth-child(2) td:not(:first-child)').map {|e| e.text.strip}
+    )
   end
 
   def test_get_permissions_should_set_css_class
@@ -326,7 +399,7 @@ class WorkflowsControllerTest < Redmine::ControllerTest
   def test_post_permissions
     WorkflowPermission.delete_all
 
-    post :permissions, :params => {
+    patch :update_permissions, :params => {
       :role_id => 1,
       :tracker_id => 2,
       :permissions => {
@@ -369,7 +442,7 @@ class WorkflowsControllerTest < Redmine::ControllerTest
   def test_post_copy_one_to_one
     source_transitions = status_transitions(:tracker_id => 1, :role_id => 2)
 
-    post :copy, :params => {
+    post :duplicate, :params => {
       :source_tracker_id => '1', :source_role_id => '2',
       :target_tracker_ids => ['3'], :target_role_ids => ['1']
     }
@@ -380,7 +453,7 @@ class WorkflowsControllerTest < Redmine::ControllerTest
   def test_post_copy_one_to_many
     source_transitions = status_transitions(:tracker_id => 1, :role_id => 2)
 
-    post :copy, :params => {
+    post :duplicate, :params => {
       :source_tracker_id => '1', :source_role_id => '2',
       :target_tracker_ids => ['2', '3'], :target_role_ids => ['1', '3']
     }
@@ -395,7 +468,7 @@ class WorkflowsControllerTest < Redmine::ControllerTest
     source_t2 = status_transitions(:tracker_id => 2, :role_id => 2)
     source_t3 = status_transitions(:tracker_id => 3, :role_id => 2)
 
-    post :copy, :params => {
+    post :duplicate, :params => {
       :source_tracker_id => 'any', :source_role_id => '2',
       :target_tracker_ids => ['2', '3'], :target_role_ids => ['1', '3']
     }
@@ -408,7 +481,7 @@ class WorkflowsControllerTest < Redmine::ControllerTest
 
   def test_post_copy_with_incomplete_source_specification_should_fail
     assert_no_difference 'WorkflowRule.count' do
-      post :copy, :params => {
+      post :duplicate, :params => {
         :source_tracker_id => '', :source_role_id => '2',
         :target_tracker_ids => ['2', '3'], :target_role_ids => ['1', '3']
       }
@@ -419,7 +492,7 @@ class WorkflowsControllerTest < Redmine::ControllerTest
 
   def test_post_copy_with_incomplete_target_specification_should_fail
     assert_no_difference 'WorkflowRule.count' do
-      post :copy, :params => {
+      post :duplicate, :params => {
         :source_tracker_id => '1', :source_role_id => '2',
         :target_tracker_ids => ['2', '3']
       }
